/*
 * Pisicuta.c
 *
 * Created: 5/22/2013 8:28:14 PM
 *  Author: Mirona
 */ 
/*---------------------------------------------------------------*/
/* 8-pin SD audio player R0.04                     (C)ChaN, 2010 */
/*---------------------------------------------------------------*/
#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include <stdio.h>
#include "diskio.h"
#include "pff.h"

#define FCC(c1,c2,c3,c4)	(((DWORD)c4<<24)+((DWORD)c3<<16)+((WORD)c2<<8)+(BYTE)c1)	/* FourCC */


/*---------------------------------------------------------*/
/* Work Area                                               */
/*---------------------------------------------------------*/

volatile BYTE FifoRi, FifoWi, FifoCt;	/* FIFO controls */

unsigned char Buff1[256];		/* Wave output FIFO */
unsigned char Buff2[256];		/* Wave output FIFO */

FATFS Fs;			/* File system object */
DIR Dir;			/* Directory object */
FILINFO Fno;		/* File information */

WORD rb;			/* Return value. Put this here to avoid bugs of avr-gcc */

uint16_t miliseconds;
uint8_t minutes, seconds, hours;
unsigned char time_string[20];

/*---------------------------------------------------------*/

static
DWORD load_header (void)	/* 0:Invalid format, 1:I/O error, >1:Number of samples */
{
	DWORD sz;


	if (pf_read(Buff1, 12, &rb)) return 1;	/* Load file header (12 bytes) */

	if (rb != 12 || LD_DWORD(Buff1+8) != FCC('W','A','V','E')) return 0;

	for (;;) {
		pf_read(Buff1, 8, &rb);			/* Get Chunk ID and size */
		if (rb != 8) return 0;
		sz = LD_DWORD(&Buff1[4]);		/* Chunk size */

		switch (LD_DWORD(&Buff1[0])) {	/* FCC */
			case FCC('f','m','t',' ') :					/* 'fmt ' chunk */
			if (sz > 100 || sz < 16) return 0;		/* Check chunk size */
			pf_read(Buff1, sz, &rb);					/* Get content */
			if (rb != sz) return 0;
			if (Buff1[0] != 1) return 0;				/* Check coding type (1) */
			if (Buff1[2] != 1 && Buff1[2] != 2) 		/* Check channels (1/2) */
			return 0;
			
			if (Buff1[14] != 8 && Buff1[14] != 16)	/* Check resolution (8/16) */
			return 0;
			
			OCR0A = (BYTE)(F_CPU/8/LD_WORD(&Buff1[4]))-1;	/* Sampling freq */
			
			break;

			case FCC('d','a','t','a') :				/* 'data' chunk (start to play) */
			return sz;

			case FCC('L','I','S','T') :				/* 'LIST' chunk (skip) */
			case FCC('f','a','c','t') :				/* 'fact' chunk (skip) */
			pf_lseek(Fs.fptr + sz);
			break;

			default :								/* Unknown chunk (error) */
			return 0;
		}
	}

	return 0;
}

volatile uint8_t read1 = 1;

volatile uint16_t counter = 0;

volatile uint8_t sync = 0;


ISR(TIMER0_COMPA_vect)
{
	if(sync == 1)
	{
		
		if(read1 == 1)
		{
			OCR1A = (unsigned char)Buff1[counter++];
			
			if(counter == 256)
			{
				read1 = 0;
				counter = 0;
			}
		}
		else if(read1 == 0)
		{
			OCR1A = (unsigned char)Buff2[counter++];
			if(counter == 256)
			{
				read1 = 1;
				counter = 0;
			}
		}
	}
}

void Timer0_init(void)
{
	TCCR0A = _BV(WGM01);
	TCCR0B = _BV(CS01);//CTC, FCPU/8
	TIMSK0 = _BV(OCIE0A);
}

void Timer1_init(void)
{
	TCCR1A = _BV(COM1A1)|_BV(COM1A0)|_BV(WGM10);
	TCCR1B = _BV(WGM12)|_BV(CS10);
}

static UINT play (const char *fn)
{
	DWORD sz;
	FRESULT res;
	unsigned char read2;

	
	uint16_t size = 256;
	
	if ((res = pf_open(fn)) == FR_OK) {
		
		Timer0_init();
		Timer1_init();
		
		sz = load_header();			/* Load file header */
		if (sz < 256) return (UINT)sz;
		
		
		FifoCt = 0; FifoRi = 0; FifoWi = 0;		/* Reset FIFO */
		pf_read(Buff1, 512 - (Fs.fptr % 512), &rb);	/* Snip sector unaligned part */
		sz -= rb;

		
		res = pf_read(Buff1, size, &rb);
		if (res != FR_OK) {
			while (FifoCt) ;
		return res; }
		
		sync = 1;
		
		do {
			
			read2 = read1;
			
			if(read1 == 1)
			res = pf_read(Buff2, size, &rb);
			else
			res = pf_read(Buff1, size, &rb);
			
			if (res != FR_OK) {
				break;
			}
			
			while(read2 == read1);
			
			} while (rb == size);	/* Repeat until all data read */
			
			sync = 0;
		}

		while (FifoCt) ;			/* Wait for FIFO empty */
		
		

		return res;
	}

	void do_meow(void)
	{
		char filename[20];

		pf_mount(&Fs);

		pf_opendir(&Dir, "miau");
		
		while (!pf_readdir(&Dir, &Fno) && Fno.fname[0])
		{
			strcpy(filename, "miau/");
			strcat(filename, Fno.fname);
			play(filename);
		}
	}


	/*-----------------------------------------------------------------------*/
	/* Main                                                                  */
	int main (void)
	{
		DDRD |= ((1<<PD5) | (1<<PD7));

		// SPI
		DDRB |= ((1<<PB7)|(1<<PB5)|(1<<PB4));
		PORTB &= ~(1<<PB6);

		// butoane pentru actiuni
		DDRA &= ~((1<<PA0) | (1<<PA1));
		PORTA |= ((1<<PA0) | (1<<PA1));
		
		DDRC |= 0xFF;
		PORTC = 0;
		
		sei();

		for (;;)
		{	
			if (pf_mount(&Fs))
			{
				continue;	/* Initialize FS */
			}
			
			while(1)
			{
				
				if(!(PINA&(1<<PA1)))
				{
					PORTC |= (1<<PD7);
					_delay_ms(500);

				} else
				
				if(!(PINA&(1<<PA0)))
				{
					do_meow();
				}
			}
		}
		
		return 1;
	}



